// Based on composite_test.cpp by Andrew Jenner

#include <stdio.h>
#define _USE_MATH_DEFINES
#include <math.h>

// 1 = Don't simulate the color burst signal (for CGA modes where bit 2 of port 0x3D8 is set).
#define MONO 0
// 1 = Raw composite output (no color or artifacting; as on monochrome composite monitors).
#define RAW_COMPOSITE 0
// Tint control (degrees)
#define TINT 0
// Saturation factor
#define SATURATION 1

int pal16[16][3]={{0,0,0},{0,0,170},{0,170,0},{0,170,170},{170,0,0},{170,0,170},{170,85,0},{170,170,170},{85,85,85},{85,85,255},{85,255,85},{85,255,255},{255,85,85},{255,85,255},{255,255,85},{255,255,255}};

int main()
{
    double Ys[1280],Is[1280],Qs[1280];
    int Rs[1280],Gs[1280],Bs[1280];
    double hue_adjust = (-(45+33)+TINT)*M_PI/180.0;

	FILE* out=fopen("out.raw","wb");
	FILE* in=fopen("in.raw","rb");

	for (int y=0;y<200;++y) {
        int rgbi = 0;
		for (int x=0;x<1280;++x) { 
			if ((x&1)==0) {
				// raw indexed RGBI input:
                rgbi = fgetc(in)&15;

				// raw 3-channel (interleaved) RGB input
				// (all colors must match the values in pal16 array):
				/*
				int triad[3];
				for (int i=0;i<3;++i) {
					triad[i] = fgetc(in);
				}
				rgbi = 0; // fallback value
				for (int i=0;i<16;++i) {/
					if ((triad[0]==pal16[i][0]) && (triad[1]==pal16[i][1]) && (triad[2]==pal16[i][2]))
						rgbi = i;
				}*/
			}

            double luma = (rgbi&8) ? 0.33 : 0;
            int color_burst[8][8]={
                {0,0,0,0,0,0,0,0}, /* D0 */
                {0,0,0,1,1,1,1,0}, /* D1 */
                {1,1,0,0,0,0,1,1}, /* D2 */
                {1,0,0,0,0,1,1,1}, /* D3 */
                {0,1,1,1,1,0,0,0}, /* D4 */
                {0,0,1,1,1,1,0,0}, /* D5 */
                {1,1,1,0,0,0,0,1}, /* D6 */
                {1,1,1,1,1,1,1,1}};/* D7 */
            double chroma;
            if (MONO)
                chroma = (rgbi!=0 ? 0.66 : 0);
            else
                chroma = (color_burst[rgbi&7][x&7] ? 0.66 : 0);
            double composite = luma + chroma;

            double s = sin(hue_adjust + (x&7)*M_PI/4.0);
            double c = cos(hue_adjust + (x&7)*M_PI/4.0);
            Ys[x]=composite;
            if (MONO) 
                Is[x]=Qs[x]=0;
            else {
                Is[x]=composite*2*c;
                Qs[x]=composite*2*s;
            }

			if (RAW_COMPOSITE)
				Rs[x]=Gs[x]=Bs[x]=composite*255.0;
			else {
				double Y=0;
				double Q=0;
				double I=0;
				for (int i=0;i<8;++i) {
					Y += Ys[x-i];
					I += Is[x-i];
					Q += Qs[x-i];
				}
				Y/=8.0;
				I/=8.0;
				Q/=8.0;
				I*=SATURATION;
				Q*=SATURATION;
				if (Y>1.0) Y=1.0; if (Y<0) Y=0;
				if (I>0.5957) I=0.5957; if (I<-0.5957) I=-0.5957;
				if (Q>0.5226) Q=0.5226; if (Q<-0.5226) Q=-0.5226;
				double R = Y + 0.9562948323208939905*I + 0.6210251254447287141*Q;
				double G = Y - 0.2721214740839773195*I - 0.6473809535176157223*Q;
				double B = Y - 1.1069899085671282160*I + 1.7046149754988293290*Q;
				int r=R*255;
				int g=G*255;
				int b=B*255;
				if (r<0) r=0; if (r>255) r=255;
				if (g<0) g=0; if (g>255) g=255;
				if (b<0) b=0; if (b>255) b=255;
				Rs[x]=r;
				Gs[x]=g;
				Bs[x]=b;
			}

            fputc(Rs[x],out);
            fputc(Gs[x],out);
            fputc(Bs[x],out);
        }
    }
    fclose(in);
    fclose(out);
    return 0;
}